/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include "jabber.h"
#include "jabprivate.h"
#include "jabstream.h"

struct _jab_typelist_str
{
    char *name;
    int node;
    int type;
};

struct _jab_typelist_str _jab_typelist[] = {
    { "normal", JABN_MSG, JABM_TYPE_NORMAL },
    { "error", JABN_MSG, JABM_TYPE_ERROR },
    { "chat", JABN_MSG, JABM_TYPE_CHAT },
    { "groupchat", JABN_MSG, JABM_TYPE_GROUPCHAT },
/*  Gurer wants it gone. 16 character waste of bandwidth or something... */
/*  { "available", JABN_PRS, JABP_TYPE_AVAILABLE }, */
    { "unavailable", JABN_PRS, JABP_TYPE_UNAVAILABLE },
    { "subscribe", JABN_PRS, JABP_TYPE_SUBSCRIBE },
    { "subscribed", JABN_PRS, JABP_TYPE_SUBSCRIBED },
    { "unsubscribe", JABN_PRS, JABP_TYPE_UNSUBSCRIBE },
    { "unsubscribed", JABN_PRS, JABP_TYPE_UNSUBSCRIBED },
    { "probe", JABN_PRS, JABP_TYPE_PROBE },
    { "get", JABN_IQ, JABQ_TYPE_GET },
    { "set", JABN_IQ, JABQ_TYPE_SET },
    { "result", JABN_IQ, JABQ_TYPE_RESULT },
    { "error", JABN_IQ, JABQ_TYPE_ERROR },
    { NULL, 0, 0 }
};

void _jab_setnode(jabn node, pool p, va_list ap);


jabn jab_newnode(int type, ...)
{
    jabn node;
    pool p;
    int sz;
    va_list ap;

    p = pool_new();
    switch(type) {
        case JABN_MSG: sz=sizeof(struct jabm_struct); break;
        case JABN_PRS: sz=sizeof(struct jabp_struct); break;
        case JABN_IQ: sz=sizeof(struct jabq_struct); break;
        default: return(NULL);
    }
    node = (jabn)pmalloc(p,sz);
    memset(node,0,sz);
    node->jn.p = p;
    node->jn.node = type;

    va_start(ap,type);
    _jab_setnode(node,p,ap);
    va_end(ap);

    return(node);
}


void jab_editnode(jabn node, ...)
{
    va_list ap;

    va_start(ap,node);
    _jab_setnode(node,node->jn.p,ap);
    va_end(ap);
}


void _jab_setnode(jabn node, pool p, va_list ap)
{
    char *arg;
    xmlnode x;

    while(1) {
        switch(va_arg(ap,int)) {
            /* common */
            case JABN_DONE:
                return;
            case JABN_ID:
                arg = va_arg(ap,char*);
                if(arg) node->jn.id=pstrdup(p,arg);
                break;
            case JABN_FROM:
                arg = va_arg(ap,char*);
                if(arg) node->jn.from=jid_new(p,arg);
                break;
            case JABN_TO:
                arg = va_arg(ap,char*);
                if(arg) node->jn.to=jid_new(p,arg);
                break;
            case JABN_TYPE:
                node->jn.type = va_arg(ap,int);
                break;
            /* message spesific */
            case JABM_BODY:
                arg = va_arg(ap,char*);
                if(arg) node->jm.body=pstrdup(p,arg);
                break;
            case JABM_SUBJECT:
                arg = va_arg(ap,char*);
                if(arg) node->jm.subject=pstrdup(p,arg);
                break;
            case JABM_THREAD:
                arg = va_arg(ap,char*);
                if(arg) node->jm.thread=pstrdup(p,arg);
                break;
            case JABM_PRIORITY:
                node->jm.priority = va_arg(ap,int);
                break;
            case JABM_X:
                x = va_arg(ap,xmlnode);
                if(x) node->jm.x = x;
            /* presence spesific */
            case JABP_STATUS:
                arg = va_arg(ap,char*);
                if(arg) node->jp.status=pstrdup(p,arg);
                break;
            case JABP_PRIORITY:
                node->jp.priority = va_arg(ap,int);
                break;
            /* iq spesific */
            case JABQ_QUERY:
                x = va_arg(ap,xmlnode);
                if(x) node->jq.query=x;
                break;
        }
    }
}


jabn jab_parsenode(xmlnode xml)
{
    jabn node;
    pool p;
    int sz,t;
    char *tmp;
    xmlnode tmpx;

    p = pool_new();

    tmp = xmlnode_get_name(xml);
    if(j_strcmp(tmp,"message")==0) {
        t = JABN_MSG;
        sz = sizeof(struct jabm_struct);
    } else if(j_strcmp(tmp,"presence")==0) {
        t = JABN_PRS;
        sz = sizeof(struct jabp_struct);
    } else if(j_strcmp(tmp,"iq")==0) {
        t = JABN_IQ;
        sz = sizeof(struct jabq_struct);
    } else
        return(NULL);

    node = (jabn)pmalloc(p,sz);
    memset(node,0,sz);
    node->jn.p = p;
    node->jn.node = t;

    /* common attributes */
    tmp = xmlnode_get_attrib(xml,"id");
    if(tmp) node->jn.id=pstrdup(p,tmp);
    tmp = xmlnode_get_attrib(xml,"from");
    if(tmp) node->jn.from=jid_new(p,tmp);
    tmp = xmlnode_get_attrib(xml,"to");
    if(tmp) node->jn.to=jid_new(p,tmp);
    tmp = xmlnode_get_attrib(xml,"type");
    /* if there isn't a 'type' attribute, uses default type (value 0) */
    if (tmp) {
        int i=0;
        while(_jab_typelist[i].name!=NULL) {
            if (_jab_typelist[i].node==t && j_strcmp(_jab_typelist[i].name,tmp)==0) {
                node->jn.type = _jab_typelist[i].type;
                break;
            }
            i++;
        }
    }

    switch(t) {
        case JABN_MSG:
            /* parse message args */
            tmpx = xmlnode_get_tag(xml,"body");
            if(tmpx) node->jm.body=pstrdup(p,xmlnode_get_data(xmlnode_get_firstchild(tmpx)));
            tmpx = xmlnode_get_tag(xml,"subject");
            if(tmpx) node->jm.subject=pstrdup(p,xmlnode_get_data(xmlnode_get_firstchild(tmpx)));
            tmpx = xmlnode_get_tag(xml,"thread");
            if(tmpx) node->jm.thread=pstrdup(p,xmlnode_get_data(xmlnode_get_firstchild(tmpx)));
            tmpx = xmlnode_get_tag(xml,"priority");
            if(tmpx) node->jm.priority=atoi(xmlnode_get_data(xmlnode_get_firstchild(tmpx)));
            tmpx = xmlnode_get_tag(xml,"x");
            if(tmpx) node->jm.x = tmpx;
            break;
        case JABN_PRS:
            /* parse presence args */
            tmpx = xmlnode_get_tag(xml,"status");
            if(tmpx) node->jp.status=pstrdup(p,xmlnode_get_data(xmlnode_get_firstchild(tmpx)));
            tmpx = xmlnode_get_tag(xml,"priority");
            if(tmpx) node->jp.priority=atoi(xmlnode_get_data(xmlnode_get_firstchild(tmpx)));
            break;
        case JABN_IQ:
            /* parse iq args */
            tmpx = xmlnode_get_tag(xml,"query");
            node->jq.query = tmpx;
            tmp = xmlnode_get_attrib(tmpx,"xmlns");
            if(tmp) node->jq.xmlns=pstrdup(p,tmp);
            break;
    }

    return(node);
}


void jab_sendnode(jabc j, jabn node)
{
    pool p;
    spool xml;
    char *buf;
    int i = 0;

    if (!j || !node) return;
    if (JAB_S_ON != j->state) return;

    p = pool_new();
    xml = spool_new(p);

    switch(node->jn.node) {
        case JABN_MSG: spool_add(xml,"<message"); break;
        case JABN_PRS: spool_add(xml,"<presence"); break;
        case JABN_IQ: spool_add(xml,"<iq"); break;
    }
    if(node->jn.id) spooler(xml," id='",strescape(p,node->jn.id),"'",xml);
    if(node->jn.from) spooler(xml," from='",strescape(p,jid_full(node->jn.from)),"'",xml);
    if(node->jn.to) spooler(xml," to='",strescape(p,jid_full(node->jn.to)),"'",xml);
    while(_jab_typelist[i].name!=NULL)
    {
        if (_jab_typelist[i].node==node->jn.node && _jab_typelist[i].type==node->jn.type)
        {
            spooler(xml," type='",_jab_typelist[i].name,"'",xml);
            break;
        }
        i++;
    }

    switch(node->jn.node) {
        case JABN_MSG:
            spool_add(xml,">");
            if(node->jm.thread)
                spooler(xml,"<thread>",strescape(p,node->jm.thread),"</thread>",xml);
            if(node->jm.subject)
                spooler(xml,"<subject>",strescape(p,node->jm.subject),"</subject>",xml);
            if(node->jm.body)
                spooler(xml,"<body>",strescape(p,node->jm.body),"</body>",xml);
            if(node->jm.x) spool_add(xml,xmlnode2str(node->jm.x));
            spool_add(xml,"</message>");
            break;
        case JABN_PRS:
            if(node->jp.type==JABP_TYPE_AVAILABLE || node->jp.type==JABP_TYPE_UNAVAILABLE) {
                char buf2[20];
                spool_add(xml,">");
                if(node->jp.status)
                    spooler(xml,"<status>",strescape(p,node->jp.status),"</status>",xml);
                    sprintf(buf2,"%d",node->jp.priority);
                    spooler(xml,"<priority>",buf2,"</priority>",xml);
                spool_add(xml,"</presence>");
            } else {
                spool_add(xml,"/>");
            }
            break;
        case JABN_IQ:
            spooler(xml,">",xml);
            if(node->jq.query) spool_add(xml,xmlnode2str(node->jq.query));
            spool_add(xml,"</iq>");
            break;
    }

    buf = spool_print(xml);
    if(buf)
    {
        jabstream_write(j,buf);
    }
    else
        fprintf(stderr, "[node.c:jab_sendnode] Empty output buffer!\n");
        
    pool_free(p);

    if(node->jn.node != JABN_MSG)
        jab_freenode(node);
}


void jab_freenode(jabn node)
{
    pool_free(node->jn.p);
}
