/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include "jabber.h"
#include "jabprivate.h"

void _jabr_addgroup(pool p, jabri item, char *group);


void jabr_request(jabc j)
{
    jabn node;
    xmlnode x;

    x = xmlnode_new_tag("query");
    xmlnode_put_attrib(x,"xmlns","jabber:iq:roster");
    node = jab_newnode(JABN_IQ,
            JABQ_TYPE, JABQ_TYPE_GET,
            JABQ_QUERY, x,
            JABQ_DONE);
    jab_sendnode(j,node);
    xmlnode_free(x);
}


void jabr_additem(jabc j, char *group, char *user, char *name)
{
    jabn node;
    xmlnode xml,tmp;

    xml = xmlnode_new_tag("query");
    xmlnode_put_attrib(xml, "xmlns", "jabber:iq:roster");

    tmp = xmlnode_insert_tag(xml,"item");
    xmlnode_put_attrib(tmp,"jid",user);
    if(name) xmlnode_put_attrib(tmp,"name",name);

    if(group) {
    tmp = xmlnode_insert_tag(tmp,"group");
    xmlnode_insert_cdata(tmp,group,-1);
    }

    node = jab_newnode(JABN_IQ,
            JABQ_TYPE, JABQ_TYPE_SET,
            JABQ_QUERY, xml,
            JABQ_DONE);
    jab_sendnode(j,node);
    xmlnode_free(xml);
}


void jabr_removeitem(jabc j, char *user)
{
    jabn node;
    xmlnode xml,itm;

    xml = xmlnode_new_tag("query");
    xmlnode_put_attrib(xml,"xmlns","jabber:iq:roster");

    itm = xmlnode_insert_tag(xml,"item");
    xmlnode_put_attrib(itm,"jid",user);
    xmlnode_put_attrib(itm,"subscription","remove");

    node = jab_newnode(JABN_IQ,
            JABQ_TYPE, JABQ_TYPE_SET,
            JABQ_QUERY, xml,
            JABQ_DONE);
    jab_sendnode(j,node);
    xmlnode_free(xml);
}


jabri jabr_updateitem(char *user, char *name)
{
    pool p;
    jabri item;

    p = pool_new();
    if(!p) return(NULL);
    item = (jabri)pmalloc(p,sizeof(_jabri));
    memset(item,0,sizeof(_jabri));
    item->next = (jabri)p;
    item->user = user;
    item->name = name;
    return(item);
}


void jabr_updategroup(jabri item, char *group)
{
    _jabr_addgroup((pool)(item->next),item,group);
}


void jabr_sendupdate(jabc j, jabri item)
{
    jabn node;
    xmlnode xml,itm,grp;
    jabrg g;

    xml = xmlnode_new_tag("query");
    xmlnode_put_attrib(xml, "xmlns", "jabber:iq:roster");

    itm = xmlnode_insert_tag(xml,"item");
    xmlnode_put_attrib(itm,"jid",item->user);
    if(item->name) xmlnode_put_attrib(itm,"name",item->name);

    g = item->groups;
    while(g) {
    grp = xmlnode_insert_tag(itm,"group");
    xmlnode_insert_cdata(grp,g->name,-1);
    g = g->next;
    }

    node = jab_newnode(JABN_IQ,
            JABQ_TYPE, JABQ_TYPE_SET,
            JABQ_QUERY, xml,
            JABQ_DONE);
    jab_sendnode(j,node);
    xmlnode_free(xml);
    pool_free((pool)(item->next));
}


void jabr_free(jabr r)
{
    pool_free(r->p);
}    


/*
** private functions
*/

void _jabr_parse(jabc j, jabq iq)
{
    pool p;
    jabr r;
    xmlnode itm,grp;
    char *tmp;
    int type=0;

    p = pool_new();
    if(!p) return;
    r = (jabr)pmalloc(p,sizeof(_jabr));
    memset(r,0,sizeof(_jabr));
    r->p = p;

    if(iq->type == JABQ_TYPE_SET)
    type = JABR_PUSH;
    else if(iq->type == JABQ_TYPE_RESULT)
    type = JABR_ROSTER;

    itm = xmlnode_get_firstchild(iq->query);
    while(itm) {
    jabri item;

    tmp = xmlnode_get_attrib(itm,"jid");
    item = (jabri)pmalloc(p,sizeof(_jabri));
    memset(item,0,sizeof(_jabri));
    item->user = pstrdup(p,tmp);
    if(r->lastitem)
        r->lastitem->next = item;
    else
        r->items = item;

    r->lastitem = item;

    tmp = xmlnode_get_attrib(itm,"name");
    item->name = pstrdup(p,tmp);

    tmp = xmlnode_get_attrib(itm,"subscription");
    if(!j_strcmp(tmp,"to"))
        item->sub = JABR_SUB_TO;
    else if(!j_strcmp(tmp,"from"))
        item->sub = JABR_SUB_FROM;
    else if(!j_strcmp(tmp,"both"))
        item->sub = JABR_SUB_BOTH;
    else if(!j_strcmp(tmp,"none"))
        item->sub = JABR_SUB_NONE;
    else
        item->sub = JABR_SUB_NDEF;
            
    tmp = xmlnode_get_attrib(itm,"ask");
    if(!j_strcmp(tmp,"subscribe"))
        item->ask = JABR_ASK_SUB;
    else if(!j_strcmp(tmp, "unsubscribe"))
        item->ask = JABR_ASK_UNSUB;
    else
        item->ask = JABR_ASK_NDEF;

    grp = xmlnode_get_firstchild(itm);
    while(grp) {
        if(j_strcmp(xmlnode_get_name(grp),"group")==0)
        _jabr_addgroup(p,item,xmlnode_get_data(xmlnode_get_firstchild(grp)));
        grp = xmlnode_get_nextsibling(grp);
    }

    itm = xmlnode_get_nextsibling(itm);
    }

    (_jabcb_roster)(j,r,type);
}


void _jabr_addgroup(pool p, jabri item, char *group)
{
    jabrg g;

    g = (jabrg)pmalloc(p,sizeof(_jabrg));
    memset(g,0,sizeof(_jabrg));
    g->name = pstrdup(p,group);
    if(item->lastgroup)
    item->lastgroup->next = g;
    else
    item->groups = g;

    item->lastgroup = g;
}
